#ifdef LOCAL
#define _GLIBCXX_DEBUG
#endif

#include <bits/stdc++.h>

using namespace std;

using ll = long long;
using ull = unsigned long long;

const int mod = 998244353;
const int root = 3;

void add (int &a, const int b)
{
    a += b;
    if (a >= mod)
        a -= mod;
}
void sub (int &a, const int b)
{
    a -= b;
    if (a < 0)
        a += mod;
}
int mult (const int a, const int b)
{
    return int(ll(a) * b % mod);
}
int powmod (int a, int pw)
{
    int res = 1;
    while (pw)
    {
        if (pw & 1)
            res = mult(res, a);
        a = mult(a, a);
        pw >>= 1;
    }
    return res;
}

void fft (vector<int> &v, const bool inv)
{
    const int n = int(v.size());
    for (int i = 1, j = 0; i < n; i++)
    {
        int bit = (n >> 1);
        while (j & bit)
        {
            j ^= bit;
            bit >>= 1;
        }
        j ^= bit;

        if (i < j)
            swap(v[i], v[j]);
    }

    for (int len = 1; len < n; len *= 2)
    {
        vector<int> w(len, 1);
        const int cur_root = powmod(root, (mod - 1) / (2 * len));
        assert(powmod(cur_root, 2 * len) == 1);
        assert(powmod(cur_root, len) != 1);

        for (int i = 1; i < len; i++)
            w[i] = mult(w[i - 1], cur_root);

        for (int i = 0; i < n; i += 2 * len)
        for (int j = 0; j < len; j++)
        {
            const int x = v[i + j];
            const int y = mult(v[i + j + len], w[j]);
            add(v[i + j], y);
            v[i + j + len] = x;
            sub(v[i + j + len], y);
        }
    }

    if (inv)
    {
        const int inv_n = powmod(n, mod - 2);
        assert(mult(n, inv_n) == 1);
        reverse(v.begin() + 1, v.end());
        for (auto &it : v)
            it = mult(it, inv_n);
    }
}

int up (int n)
{
    int ans = 1;
    while (ans < n)
        ans *= 2;
    return ans;
}

vector<int> prod (vector<int> l, vector<int> r)
{
    const int len = up(int(l.size() + r.size()));

    l.resize(len, 0);
    r.resize(len, 0);
    fft(l, false);
    fft(r, false);

    for (int i = 0; i < len; i++)
        l[i] = mult(l[i], r[i]);

    fft(l, true);
    while (!l.empty() && l.back() == 0)
        l.pop_back();
    return l;
}

/// Uses FFT, as Vanya suggested in the chat.
void solve_test(istream &cin = std::cin, ostream &cout = std::cout)
{
    vector<int> cnt(3);
    for (auto &it : cnt)
        cin >> it;

    const int bnd = accumulate(cnt.begin(), cnt.end(), 0) + 1;
    vector<int> fact(bnd, 1), inv(bnd, 1), ifact(bnd, 1);
    for (int i = 2; i < bnd; i++)
    {
        fact[i] = mult(i, fact[i - 1]);
        inv[i] = mod - mult(mod / i, inv[mod % i]);
        assert(mult(i, inv[i]) == 1);
        ifact[i] = mult(inv[i], ifact[i - 1]);
        assert(mult(fact[i], ifact[i]) == 1);
    }

    vector<vector<int>> poly(3);
    for (int i = 0; i < 3; i++)
    {
        poly[i].assign(cnt[i] + 1, 0);
        for (int seg = 1; seg <= cnt[i]; seg++)
        {
            int cur = mult(ifact[seg - 1], ifact[cnt[i] - seg]);
            cur = mult(cur, fact[cnt[i] - 1]);
            cur = mult(cur, ifact[seg]);

            if ((cnt[i] - seg) % 2 == 1)
                cur = mult(cur, mod - 1);
            poly[i][seg] = cur;
        }

        if (cnt[i] == 0)
            poly[i][0] = 1;
    }

    auto res = prod(prod(poly[0], poly[1]), poly[2]);
    assert(int(res.size()) <= bnd);

    int ans = 0;
    for (int i = 0; i < int(res.size()); i++)
        add(ans, mult(fact[i], res[i]));

    cout << ans << endl;
}

void solve(istream &cin = std::cin, ostream &cout = std::cout)
{
    solve_test(cin, cout);
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout << fixed;

#ifdef LOCAL
    auto st = clock();

    ifstream fin("input.txt");
    ofstream fout("output.txt");

    do
    {
        solve(fin, fout);

        fout << "===" << endl;

        string str;
        while (getline(fin, str) && str != string(max(1, (int) str.size()), '='));
    }
    while (fin);

    cout << setprecision(6) << "clock: " << double(clock() - st) / CLOCKS_PER_SEC << endl;
#else
    solve();
#endif

    return 0;
}
