#ifdef LOCAL
#define _GLIBCXX_DEBUG
#endif

#include <bits/stdc++.h>

using namespace std;

using ll = long long;
using ull = unsigned long long;

const int mod = 998244353;

void add (int &a, const int b)
{
    a += b;
    if (a >= mod)
        a -= mod;
}
void sub (int &a, const int b)
{
    a -= b;
    if (a < 0)
        a += mod;
}
int mult (const int a, const int b)
{
    return int(ll(a) * b % mod);
}

/// Based on the formula for f(a, b, c), which can be obtained in multiple ways:
/// solving linear systems over generating functions, normal combinatorial case
/// handling, doing inclusion-exclusion, e.t.c.
void solve_test(istream &cin = std::cin, ostream &cout = std::cout)
{
    int r, g, b;
    cin >> r >> g >> b;

    const int bnd = r + g + b + 1;
    vector<int> fact(bnd, 1), inv(bnd, 1), ifact(bnd, 1);
    for (int i = 2; i < bnd; i++)
    {
        fact[i] = mult(i, fact[i - 1]);
        inv[i] = mod - mult(mod / i, inv[mod % i]);
        assert(mult(i, inv[i]) == 1);
        ifact[i] = mult(inv[i], ifact[i - 1]);
        assert(mult(fact[i], ifact[i]) == 1);
    }

    vector<int> pw2(bnd, 1);
    for (int i = 1; i < bnd; i++)
        pw2[i] = mult(2, pw2[i - 1]);

    auto f = [&] (array<int, 3> cnt)
    {
        int ans = 0;

        const int mn = *min_element(cnt.begin(), cnt.end());
        for (int triple = 0; triple <= mn; triple++)
        {
            array<int, 3> shifted = cnt;
            for (auto &it : shifted)
                it -= triple;

            const int sum = accumulate(shifted.begin(), shifted.end(), 0);
            if (sum % 2 != 0)
                continue;
            const int moves = (sum / 2);

            int cur = pw2[triple];
            cur = mult(cur, fact[triple + moves]);
            cur = mult(cur, ifact[triple]);
            for (const auto it : shifted)
                cur = mult(cur, (it > moves ? 0 : ifact[moves - it]));

            add(ans, cur);
        }

        return ans;
    };

    int ans = 0;
    for (int sr = 0; sr <= 1; sr++)
    for (int sg = 0; sg <= 1; sg++)
    for (int sb = 0; sb <= 1; sb++)
        add(ans, f(array<int, 3>{r - sr, g - sg, b - sb}));

    cout << ans << endl;
}

void solve(istream &cin = std::cin, ostream &cout = std::cout)
{
    solve_test(cin, cout);
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout << fixed;

#ifdef LOCAL
    auto st = clock();

    ifstream fin("input.txt");
    ofstream fout("output.txt");

    do
    {
        solve(fin, fout);

        fout << "===" << endl;

        string str;
        while (getline(fin, str) && str != string(max(1, (int) str.size()), '='));
    }
    while (fin);

    cout << setprecision(6) << "clock: " << double(clock() - st) / CLOCKS_PER_SEC << endl;
#else
    solve();
#endif

    return 0;
}
