// Author: Ivan Kazmenko (gassa@mail.ru)
// Generator for problem "count-operations"
module gen;
import std.algorithm;
import std.conv;
import std.exception;
import std.format;
import std.random;
import std.range;
import std.stdio;
import std.typecons;

immutable string testFormat = "%03d";
enum generateAnswers = false;

immutable int minV =       0;
immutable int maxV = 10 ^^ 9;
immutable int minK =       0;
immutable int maxK = 10 ^^ 5;

alias Pair = Tuple !(int, q{x}, int, q{y});

struct Test
{
	int n;
	Pair [] a;

	string comment;

	@property auto k () const
	{
		return a.length.to !(int);
	}

	void validate () const
	{
		enforce (minV < n && n <= maxV);
		enforce (minK <= k && k <= maxK);
		foreach (ref p; a)
		{
			enforce (minV <= p.x && p.x < n);
			enforce (minV <= p.y && p.y < n);
		}
	}

	void print (File f) const
	{
		f.writefln !("for (int i = 0; i < %s; i++) {") (n);
		foreach (ref p; a)
		{
			f.writefln !("    if (i == %s) i = %s;") (p.x, p.y);
		}
		f.writefln !("    work (i);");
		f.writefln !("}");
	}

	string log () const
	{
		return format !("%s (n = %s, k = %s))") (comment, n, k);
	}
}

Test [] tests;

void printAllTests ()
{
	foreach (testNumber, test; tests)
	{
		test.validate ();
		auto testString = format (testFormat, testNumber + 1);
		auto f = File (testString, "wt");
		test.print (f);
		static if (generateAnswers)
		{
			auto g = File (testString ~ ".a", "wt");
			test.printAnswer (g);
		}
		writeln (testString, ": ", test.log ());
	}
}

Mt19937 rndLocal;
T rndValue (T) () {return uniform !(T) (rndLocal);}
T rndValue (T) (T lim) {return uniform (cast (T) (0), lim, rndLocal);}
T rndValue (string boundaries = "[)", T) (T lo, T hi)
    {return uniform !(boundaries) (lo, hi, rndLocal);}
auto rndChoice (R) (R r) {return r[rndValue (cast (int) (r.length))];}

void rndShuffle (R) (R r)
{
	auto len = r.length.to !(int);
	foreach (i; 0..len)
	{
		int k = rndValue (i, len);
		r.swapAt (i, k);
	}
}

Pair [] genRandom (int k, int lo, int hi)
{
	Pair [] res;
	foreach (i; 0..k)
	{
		auto x = rndValue !("[]") (lo, hi);
		auto y = rndValue !("[]") (lo, hi);
		res ~= Pair (x, y);
	}
	return res;
}

Pair [] genRandomForward (int k, int lo, int hi)
{
	Pair [] res;
	foreach (i; 0..k)
	{
		auto x = rndValue !("[]") (lo, hi);
		auto y = rndValue !("[]") (lo, hi);
		if (x > y)
		{
			swap (x, y);
		}
		res ~= Pair (x, y);
	}
	return res;
}

Pair [] genRandomPoints (int k, int lo, int hi)
{
	Pair [] res;
	foreach (i; 0..k)
	{
		auto x = rndValue !("[]") (lo, hi);
		res ~= Pair (x, x);
	}
	return res;
}

Pair [] genRandomLoops (int k, int lo, int hi, int point, int p, int q)
{
	Pair [] res;
	enforce (k % 2 == 0);
	foreach (i; 0..k / 2)
	{
		auto x = rndValue !("[]") (lo, hi);
		res ~= Pair (x, point);
		res ~= Pair (point, x);
	}
	foreach (i; 0..k / 2 - 2)
	{
		if (rndValue (p + q) < p)
		{
			reverse (res[i * 2 + 1..i * 2 + 4]);
		}
	}
	return res;
}

Pair [] genRandomSegments (int k, int lo, int hi, int space, int inv)
{
	int [] a;
	foreach (i; 0..k * 2)
	{
		a ~= rndValue !("[]") (lo, hi - k * 2 * space);
	}
	sort (a);
	foreach (i; 0..k * 2)
	{
		a[i] += i * space;
	}
	immutable int limit = 10;
	foreach (j; 0..inv)
	{
		auto i = rndValue (0, k - limit) * 2 + 1;
		auto d = rndValue (1, limit) * 2;
		reverse (a[i..i + d]);
	}
	Pair [] res;
	foreach (i; 0..k)
	{
		res ~= Pair (a[0], a[1]);
		a = a[2..$];
	}
	rndShuffle (res);
	return res;
}

Pair [] genRandomRoute (int k, int lo, int hi, int space, int inv)
{
	int [] a;
	foreach (i; 0..k * 2)
	{
		a ~= rndValue !("[]") (lo, hi - k * 2 * space);
	}
	sort (a);
	foreach (i; 0..k * 2)
	{
		a[i] += i * space;
	}
	foreach (j; 0..inv)
	{
		auto i = rndValue (0, k * 2 - 1);
		swap (a[i], a[i + 1]);
	}
	auto p = iota (k + 1).array;
	rndShuffle (p[1..$ - 1]);
	Pair [] res;
	foreach (i; 0..k)
	{
		res ~= Pair (a[p[i] * 2 + 0], a[p[i + 1] * 2 - 1]);
	}
	rndShuffle (res);
	return res;
}

void main ()
{
	rndLocal.seed (927_716_634);

	tests ~= Test (3, [], "example test 1");
	tests ~= Test (5, [Pair (3, 1), Pair (1, 3), Pair (3, 4)],
	    "example test 2");

	tests ~= Test (1, [], "minimal test 1");
	tests ~= Test (1, [Pair (0, 0)], "minimal test 2");
	tests ~= Test (2, [Pair (1, 0)], "minimal test 3");
	tests ~= Test (2, [Pair (1, 0), Pair (1, 1)], "minimal test 4");
	tests ~= Test (2, [Pair (1, 1), Pair (1, 0)], "minimal test 5");
	tests ~= Test (2, [Pair (1, 0), Pair (0, 1)], "minimal test 6");
	tests ~= Test (3, [Pair (1, 0), Pair (0, 1), Pair (2, 0), Pair (0, 2)],
	    "minimal test 7");
	tests ~= Test (3, [Pair (2, 0), Pair (0, 1), Pair (1, 0), Pair (0, 2)],
	    "minimal test 8");
	tests ~= Test (3, [Pair (2, 0), Pair (1, 0), Pair (0, 1), Pair (0, 2)],
	    "minimal test 9");
	tests ~= Test (3, [Pair (2, 0), Pair (1, 0), Pair (0, 2), Pair (0, 1)],
	    "minimal test 10");
	tests ~= Test (99, [Pair (12, 98)],
	    "small test 1");
	tests ~= Test (99, [Pair (10, 50), Pair (60, 20)],
	    "small test 2");
	tests ~= Test (99, [Pair (10, 50), Pair (60, 20), Pair (30, 70)],
	    "small test 3");
	tests ~= Test (99, [Pair (10, 60), Pair (50, 20)],
	    "small test 4");
	tests ~= Test (99, [Pair (60, 20), Pair (20, 60),
	    Pair (20, 70), Pair (65, 64)],
	    "small test 5");
	tests ~= Test (99, [Pair (60, 20), Pair (20, 70),
	    Pair (20, 60), Pair (65, 64)],
	    "small test 6");
	tests ~= Test (99, [Pair (60, 20), Pair (20, 60),
	    Pair (21, 70), Pair (65, 64)],
	    "small test 7");

	tests ~= Test (10 ^^ 9, [], "medium test 1");
	tests ~= Test (10 ^^ 9, [Pair (555_555_555, 555_555_555)],
	    "medium test 2");
	tests ~= Test (10 ^^ 9, [Pair (555_555_555, 555_555_554)],
	    "medium test 3");
	tests ~= Test (10 ^^ 9, [Pair (123_456_789, 987_654_321)],
	    "medium test 4");

	alias Bounds = Tuple !(int, q{num}, int, q{lo}, int, q{hi},
	    string, q{name});
	foreach (b; only (Bounds (10 ^^ 2, 0, 10 ^^ 4, "medium"),
	    Bounds (maxK, 0, maxV, "large")))
	{
		tests ~= Test (b.hi, genRandom (b.num, b.lo, b.hi - 1),
		    format !("%s random test 1") (b.name));
		tests ~= Test (b.hi, genRandomForward (b.num, b.lo, b.hi - 1),
		    format !("%s forward test 1") (b.name));
		tests ~= Test (b.hi, genRandom (b.num, b.hi / 3, b.hi / 3),
		    format !("%s singular test 1") (b.name));
		tests ~= Test (b.hi, genRandomPoints (b.num, b.lo, b.hi - 1),
		    format !("%s points test 1") (b.name));
		tests ~= Test (b.hi, genRandomLoops
		    (b.num & ~1, b.lo, b.hi - 1, b.hi / 3, 0, 1),
		    format !("%s loops test 1") (b.name));
		tests ~= Test (b.hi, genRandomLoops
		    (b.num & ~1, b.lo, b.hi - 1, b.hi / 3, 1, 5),
		    format !("%s loops test 2") (b.name));
		tests ~= Test (b.hi, genRandomSegments
		    (b.num, b.lo, b.hi - 1, 1, 0),
		    format !("%s segments test 1") (b.name));
		tests ~= Test (b.hi, genRandomSegments
		    (b.num, b.lo, b.hi - 1, 1, 5),
		    format !("%s segments test 2") (b.name));
		tests ~= Test (b.hi, genRandomSegments
		    (b.num, b.lo, b.hi - 1, 0, 1),
		    format !("%s segments test 3") (b.name));
		tests ~= Test (b.hi, genRandomSegments
		    (b.num, b.hi - b.num * 4, b.hi - 1, 0, 7),
		    format !("%s segments test 4") (b.name));
		tests ~= Test (b.hi, genRandomSegments
		    (b.num, b.hi - b.num * 4, b.hi - 1, 0, b.num / 8),
		    format !("%s segments test 5") (b.name));
		tests ~= Test (b.hi, genRandomRoute
		    (b.num, b.lo, b.hi - 1, 1, 0),
		    format !("%s route test 1") (b.name));
		tests ~= Test (b.hi, genRandomRoute
		    (b.num, b.lo, b.hi - 1, 1, 1),
		    format !("%s route test 2") (b.name));
		tests ~= Test (b.hi, genRandomRoute
		    (b.num, b.lo, b.hi - 1, 1, b.num / 5),
		    format !("%s route test 3") (b.name));
	}

	printAllTests ();
}
