#include "testlib.h"
#include <bits/stdc++.h>
using namespace std;

const int min_tests = 1;
const int max_tests = int(2e6);

constexpr int camel_moves = 8;
const array<int, camel_moves> camel_dx = {+3, +3, +1, +1, -1, -1, -3, -3};
const array<int, camel_moves> camel_dy = {+1, -1, +3, -3, +3, -3, +1, -1};

int main (int argc, char* argv[])
{
    registerValidation(argc, argv);

    int t = inf.readInt(min_tests, max_tests, "test_cnt");
    inf.readEoln();

    const vector<string> names = {"white king", "black king", "camel", "wazir"};

    for (int i = 0; i < t; i++)
    {
        setTestCase(i + 1);
        
        vector<pair<int, int>> pos(4);

        for (int j = 0; j < 4; j++)
        {
            const string cur = inf.readWord("[a-h][1-8]", format("position of %s", names[j].c_str()));
            /// Should hold automatically, but let's check them anyway.
            ensure(cur.size() == 2U);
            ensure('a' <= cur[0] && cur[0] <= 'h');
            ensure('1' <= cur[1] && cur[1] <= '8');

            pos[j] = pair<int, int>(int(cur[0] - 'a'), int(cur[1] - '1'));
            inf.readSpace();
        }

        const char who_moves = inf.readChar();
        ensure(who_moves == 'w' || who_moves == 'b');

        for (int i = 0; i < 4; i++)
        for (int j = 0; j < i; j++)
            ensuref(pos[i] != pos[j], "%s and %s are at the same position",
                    names[i].c_str(), names[j].c_str());

        if (abs(pos[0].first - pos[1].first) <= 1 &&
            abs(pos[0].second - pos[1].second) <= 1)
            ensuref(false, "kings attack each other");

        vector<vector<bool>> attacked(8, vector<bool>(8));

        auto is_valid = [&] (const int x, const int y)
        {
            return 0 <= x && x < 8 && 0 <= y && y < 8;
        };
        auto make_attacked = [&] (const int x, const int y)
        {
            if (is_valid(x, y))
                attacked[x][y] = true;
        };

        for (int dx = -1; dx <= +1; dx++)
        for (int dy = -1; dy <= +1; dy++)
        if (abs(dx) != 0 || abs(dy) != 0)
            make_attacked(pos[0].first + dx, pos[0].second + dy);

        for (int d = 0; d < camel_moves; d++)
            make_attacked(pos[2].first + camel_dx[d], pos[2].second + camel_dy[d]);

        for (int dx = -1; dx <= +1; dx++)
        for (int dy = -1; dy <= +1; dy++)
        if (abs(dx) + abs(dy) == 1)
            make_attacked(pos[3].first + dx, pos[3].second + dy);

        int poss_moves = 0;

        for (int dx = -1; dx <= +1; dx++)
        for (int dy = -1; dy <= +1; dy++)
        if (abs(dx) != 0 || abs(dy) != 0)
        {
            const int nx = pos[1].first + dx;
            const int ny = pos[1].second + dy;
            if (is_valid(nx, ny) && !attacked[nx][ny])
                poss_moves++;
        }

        const bool is_checked = attacked[pos[1].first][pos[1].second];
        const bool no_moves = (poss_moves == 0);

        if (is_checked)
            ensuref(who_moves == 'b', "Black is checked (and possibly mated), but it's White's move");

        if (who_moves == 'b' && no_moves)
        {
            if (is_checked)
                ensuref(false, "Black is mated");
            else
                ensuref(false, "Black is stalemated");
        }

        inf.readEoln();
    }

    inf.readEof();
}

