use self::random::Xoshiro256SS;
pub use self::template::*;
use std::env;
use std::io::{stdin, stdout, BufWriter, Write};

pub fn solve(_scan: &mut impl Scanner, cout: &mut impl Write) {
    let mut rng = Xoshiro256SS::new(736);

    let args = env::args().collect::<Vec<String>>();

    let cnt = args[1].parse::<usize>().unwrap();
    let max_tests = args[2].parse::<usize>().unwrap();

    const MAX_N: u64 = 1e18 as u64;

    let mut tests = vec![(MAX_N, MAX_N)];

    for i in 1..max_tests {
        tests.push((i as u64, MAX_N));
    }

    for t in (1..=1414213565).rev() {
        if tests.len() >= cnt / 2 {
            break;
        }

        let n = (t * (t - 1) / 2 + t + 1).clamp(1, MAX_N);

        tests.push((1, n));
        tests.push((1, n - 1));
    }
    for t in 1.. {
        if tests.len() >= cnt.saturating_sub(700) {
            break;
        }

        let n = (t * (t - 1) / 2 + t + 1).clamp(1, MAX_N);

        tests.push((1, n));
        tests.push((1, n - 1));
    }

    while tests.len() < cnt.saturating_sub(36) {
        tests.push((rng.gen_range(1..=MAX_N), MAX_N));
    }
    while tests.len() < cnt.saturating_sub(30) {
        tests.push((1, rng.gen_range(1..1e18 as u64)));
    }
    while tests.len() < cnt.saturating_sub(20) {
        tests.push((rng.gen_range(1..1e18 as u64), rng.gen_range(1..1e18 as u64)));
    }
    while tests.len() < cnt.saturating_sub(10) {
        tests.push((rng.gen_range(1..60), rng.gen_range(1..1e18 as u64)));
    }
    while tests.len() < cnt {
        tests.push((rng.gen_range(1..30), rng.gen_range(1..1e18 as u64)));
    }

    rng.shuffle(&mut tests);

    writeln!(cout, "{}", tests.len()).unwrap();

    for (k, n) in tests {
        writeln!(cout, "{} {}", n, k).unwrap();
    }
}

#[allow(dead_code)]
fn main() {
    solve(
        &mut IOScanner::new(stdin().lock()),
        &mut BufWriter::new(stdout().lock()),
    );
}

pub mod template {
    use std::io::BufRead;
    use std::str::FromStr;

    pub trait Scanner {
        fn checked_next_line(&mut self) -> Option<String>;

        fn checked_next<T: FromStr>(&mut self) -> Option<T>;

        #[allow(dead_code)]
        fn next<T: FromStr>(&mut self) -> T;

        fn seek(&mut self) -> bool;
    }

    pub struct IOScanner<ReadType: BufRead> {
        read: ReadType,
        buffer: Vec<String>,
    }

    impl<ReadType: BufRead> Scanner for IOScanner<ReadType> {
        fn checked_next_line(&mut self) -> Option<String> {
            self.buffer.clear();
            let mut input = String::new();
            if self.read.read_line(&mut input).expect("Failed read") > 0 {
                Some(input)
            } else {
                None
            }
        }

        fn checked_next<T: FromStr>(&mut self) -> Option<T> {
            loop {
                if let Some(token) = self.buffer.pop() {
                    return Some(token.parse().ok().expect("Failed parse"));
                }
                if let Some(input) = self.checked_next_line() {
                    self.buffer = input.split_whitespace().rev().map(String::from).collect();
                } else {
                    return None;
                }
            }
        }

        fn next<T: FromStr>(&mut self) -> T {
            self.checked_next().expect("Read after EOF")
        }

        fn seek(&mut self) -> bool {
            let mut input = String::new();

            while let Ok(cnt) = self.read.read_line(&mut input) {
                if cnt == 0 {
                    break;
                }
                let line = input.trim();
                if !line.is_empty() && line.matches("=").count() == line.len() {
                    return true;
                }
                input.clear();
            }

            false
        }
    }

    impl<ReadType: BufRead> IOScanner<ReadType> {
        pub fn new(read: ReadType) -> IOScanner<ReadType> {
            IOScanner {
                read,
                buffer: Vec::<String>::default(),
            }
        }
    }

    pub(crate) mod random {
        use std::collections::Bound;
        use std::ops::RangeBounds;

        #[derive(Debug, Clone, Ord, PartialOrd, Eq, PartialEq, Hash)]
        pub struct Xoshiro256SS {
            state: [u64; 4],
        }

        #[allow(dead_code)]
        impl Xoshiro256SS {
            fn splitmix64(state: &mut u64) -> u64 {
                *state = state.wrapping_add(0x9E3779B97f4A7C15u64);
                let mut result = state.clone();
                result = (result ^ (result >> 30)).wrapping_mul(0xBF58476D1CE4E5B9);
                result = (result ^ (result >> 27)).wrapping_mul(0x94D049BB133111EB);
                return result ^ (result >> 31);
            }

            pub fn new(mut seed: u64) -> Self {
                let mut state = [0; 4];

                for it in &mut state {
                    *it = Self::splitmix64(&mut seed);
                }

                Self { state }
            }

            pub fn gen(&mut self) -> u64 {
                let s = &mut self.state;
                let result = s[1].wrapping_mul(5).rotate_left(7).wrapping_mul(9);
                let t = s[1] << 17;

                s[2] ^= s[0];
                s[3] ^= s[1];
                s[1] ^= s[2];
                s[0] ^= s[3];

                s[2] ^= t;
                s[3] = s[3].rotate_left(45);

                return result;
            }

            pub fn gen_modulo(&mut self, modulo: u64) -> u64 {
                self.gen() % modulo
            }

            pub fn gen_range(&mut self, range: impl RangeBounds<u64>) -> u64 {
                if range.start_bound() == Bound::Unbounded && range.end_bound() == Bound::Unbounded
                {
                    return self.gen();
                }

                let start = match range.start_bound() {
                    Bound::Included(x) => *x,
                    Bound::Excluded(x) => x + 1,
                    Bound::Unbounded => 0,
                };

                let len = match range.end_bound() {
                    Bound::Included(x) => x.wrapping_add(1),
                    Bound::Excluded(x) => *x,
                    Bound::Unbounded => 0,
                }
                .wrapping_sub(start);

                self.gen_modulo(len) + start
            }

            pub fn shuffle<T>(&mut self, arr: &mut [T]) {
                for i in 1..arr.len() {
                    arr.swap(i, self.gen_modulo((i + 1) as u64) as usize);
                }
            }
        }
    }
}
