use std::io;
use io::{BufRead, Write};
use std::str::FromStr;

#[cfg(dprintln)]
#[allow(unused_macros)]
macro_rules! dprintln { ($( $args:expr ),*) => { eprintln!( $( $args ),* ); } }

#[cfg(not(dprintln))]
#[allow(unused_macros)]
macro_rules! dprintln { ($( $args:expr ),*) => { } }

use std::ops::{Mul, Sub};

#[derive(Debug, Copy)]
struct Point<T> {
  x: T,
  y: T,
}

impl<T: Clone> Clone for Point<T> {
  fn clone(&self) -> Self {
    Self {
      x: self.x.clone(),
      y: self.y.clone(),
    }
  }
}

impl<T: Sub<Output = T>> Sub for Point<T> {
  type Output = Self;
  fn sub(self, other: Point<T>) -> Point<T> {
    Self {
      x: self.x - other.x,
      y: self.y - other.y,
    }
  }
}

impl<T: Clone + Sub<Output = T> + Mul<Output = T>> Point<T> {
  fn cross_product(&self, other: &Self) -> T {
    self.x.clone() * other.y.clone() - self.y.clone() * other.x.clone()
  }
}

struct Scanner<I> {
  it: I,
}

type PI = Point<i32>;

impl<I: std::iter::Iterator<Item=String>> Scanner<I> {
  fn next<A:FromStr>(&mut self) -> A {
    if let Ok(x) = self.it.next().unwrap().parse::<A>() {
      x
    } else {
      panic!("parse failed")
    }
  }
  fn nextp(&mut self) -> PI {
    let x = self.next::<i32>();
    let y = self.next::<i32>();
    PI {x, y}
  }
}

fn solve(a: &[PI], b: &[PI]) -> i32 {
  let mut res = 0;
  for (i, u) in a.iter().enumerate() {
    for v in a.iter().skip(i+1) {
      let v1 = *v - *u;
      for w in b {
        let v2 = *w - *u;
        res = res.max(v1.cross_product(&v2).abs());
      }
    }
  }
  res
}

fn main() -> io::Result<()> {
  let stdin = io::stdin();
  let it = stdin
    .lock()
    .lines()
    .flat_map(|s| {
       s.unwrap().split_whitespace()
        .map(|s| s.to_owned())
        .collect::<Vec<_>>()
        .into_iter()
      });
  let mut r = Scanner{it};
  let stdout = io::stdout();
  let mut o = io::BufWriter::with_capacity(1 << 10, stdout.lock());
  let n = r.next::<usize>();
  let mut a = Vec::new();
  let mut b = Vec::new();
  for _ in 0 .. n {
    let p = r.nextp();
    if p.y == 0 {
      a.push(p);
    } else {
      b.push(p);
    }
  }
  let res = solve(&a, &b);
  let u = res / 2;
  let v = res % 2;
  write!(o, "{}", u)?;
  if v != 0 {
    write!(o, ".5")?;
  }
  writeln!(o)?;
  o.flush()
}
