#include "testlib.h"
#include <array>
#include <algorithm>
#include <cmath>
#include <cstring>
#include <iostream>
#include <string>
#include <utility>
#include <vector>
 
using namespace std;
 
using Subtest = pair<string, unsigned long long>;
const int MAX_TOTAL_L = 100'000;
const unsigned long long MAX_K = 1'000'000'000'000'000'000ULL;
 
struct Test {
  vector<Subtest> a;
  Test() : a(vector<Subtest>()) {}
  Test (vector<Subtest> _a) : a(_a) {}
  void out() const {
    cout << a.size() << "\n";
    for (auto const &p: a) {
      cout << p.first << " " << p.second << "\n";
    }
  }
};
 
string digits(int n) {
  string z;
  for (int i = 1; i <= n; ++i) {
    z += (char) (48 + i);
  }
  return z;
}
 
unsigned long long count_inversions(const string &s) {
  array<int, 9> a;
  fill(a.begin(), a.end(), 0);
  unsigned long long res = 0;
  const int n = s.length();
  for (int i = n - 1; i >= 0; i--) {
    int k = (int) (s[i] - '1');
    for (int j = 0; j < k; ++j) {
      res += a[j];
    }
    a[k]++;
  }
  return res;
}
 
Test rand1(int max_nt, int sigma, int maxl, unsigned long long maxk, int order = 0) {
  vector<Subtest> a;
  auto ab = digits(sigma);
  int total_l = 0;
  for (int i = 0; i < max_nt; i++) {
    int ml = min(maxl, MAX_TOTAL_L - total_l);
    if (ml <= 0) {
      break;
    }
    const int l = rnd.next(1, ml);
    total_l += l;
    string ptr = format("[%s]{%d}", ab.c_str(), l);
    auto s = rnd.next(ptr);
    if (order == 1) {
      sort(s.begin(), s.end());
    }
    if (order == 2) {
      sort(s.rbegin(), s.rend());
    }
    if (!order && !maxk) {
      maxk = count_inversions(s);
    }
    a.push_back({s, rnd.next(0ULL, maxk)});
  }
  return { a };
}
 
Test rand_maxk(int max_nt, int sigma, int maxl) {
  vector<Subtest> a;
  auto ab = digits(sigma);
  int total_l = 0;
  for (int i = 0; i < max_nt; i++) {
    int ml = min(maxl, MAX_TOTAL_L - total_l);
    if (ml <= 0) {
      break;
    }
    const int l = rnd.next(1, ml);
    total_l += l;
    string ptr = format("[%s]{%d}", ab.c_str(), l);
    auto s = rnd.next(ptr);
    a.push_back({s, MAX_K});
  }
  return { a };
}
 
Test one_subtest(string s, unsigned long long k) {
  vector<Subtest> a;
  a.push_back({ s, k });
  return { a };
}
 
Test single(int sigma, int l, unsigned long long k) {
  auto ab = digits(sigma);
  string ptr = format("[%s]{%d}", ab.c_str(), l);
  return one_subtest(rnd.next(ptr), k);
}
 
Test almost_reversed_sorted(int swaps, int sigma, int l, double min_k_ratio) {
  auto ab = digits(sigma);
  string ptr = format("[%s]{%d}", ab.c_str(), l);
  auto s = rnd.next(ptr);
  sort(s.rbegin(), s.rend());
  for(int o = 0; o < swaps; o++) {
    int i = rnd.next(0, l - 1);
    int j = rnd.next(0, l - 1);
    swap(s[i], s[j]);
  }
  const auto inv = count_inversions(s);
  const unsigned long long min_k = (unsigned long long) (inv * min_k_ratio + 0.5);
  const unsigned long long k = rnd.next(min_k, inv);
  return one_subtest(s, k);
}
 
int main(int argc, char* argv[]) {
  registerGen(argc, argv, 1);
  Test t;
  int k = 2;
  if (!strcmp(argv[1], "rand")) {
    int order = 0;
    int max_nt = atoi(argv[k++]);
    int sigma = atoi(argv[k++]);
    int maxl = atoi(argv[k++]);
    unsigned long long maxk = atoll(argv[k++]);
    if (k < argc) {
      if (!strncmp(argv[k], "asc", 3)) {
        order = 1;
      }
      if (!strncmp(argv[k], "rev", 3) || !strncmp(argv[k], "desc", 4)) {
        order = 2;
      }
      ensure(order != 0);
    }
    t = rand1(max_nt, sigma, maxl, maxk, order);
  } else if (!strcmp(argv[1], "maxk")) {
    int max_nt = atoi(argv[k++]);
    int sigma = atoi(argv[k++]);
    int maxl = atoi(argv[k++]);
    t = rand_maxk(max_nt, sigma, maxl);
  } else if (!strcmp(argv[1], "single")) {
    int sigma = atoi(argv[k++]);
    int l = atoi(argv[k++]);
    unsigned long long ops = atoll(argv[k++]);
    t = single(sigma, l, ops);
  } else if (!strcmp(argv[1], "hand")) {
    unsigned long long ops = atoll(argv[k++]);
    string s = "";
    while (k + 2 <= argc) {
      string c = argv[k];
      int rep = atoi(argv[k+1]);
      while (--rep >= 0) {
        s += c;
      }
      k += 2;
    }
    ensure(k == argc);
    t = one_subtest(s, ops);
  } else if (!strcmp(argv[1], "almostrevsorted")) {
    int swaps = atoi(argv[k++]);
    int sigma = atoi(argv[k++]);
    int l = atoi(argv[k++]);
    double min_k_ratio = atof(argv[k++]);
    t = almost_reversed_sorted(swaps, sigma, l, min_k_ratio);
  } else {
    return 1;
  }
  t.out();
  return 0;
}