use io::{BufRead, Write};
use std::collections::HashSet;
use std::io;
use std::str::FromStr;

struct Scanner<I> {
  it: I,
}

impl<I: std::iter::Iterator<Item = String>> Scanner<I> {
  fn next<A: FromStr>(&mut self) -> A {
    if let Ok(x) = self.it.next().unwrap().parse::<A>() {
      x
    } else {
      panic!("parse failed")
    }
  }
}

fn unpack(x: i32, n: usize) -> Vec<i32> {
  let mut b = Vec::with_capacity(n);
  let mut y = x;
  for _ in 0..n {
    b.push(y % 9);
    y /= 9;
  }
  b.reverse();
  b
}

fn solve(a: &[u8], k: i64) -> String {
  let n = a.len();
  assert!(n <= 9);
  let mult = (0..n).rev().map(|k| 9i32.pow(k as u32)).collect::<Vec<_>>();
  let start = a
    .iter()
    .zip(mult.iter())
    .map(|(p, q)| *p as i32 * *q)
    .sum::<i32>();
  let mut h = HashSet::new();
  let mut q = Vec::new();
  q.push(start);
  let mut res = start;
  for _steps in 1..=k {
    let mut w = Vec::new();
    for x in q {
      let b = unpack(x, n);
      for j in 1..n {
        let i = j - 1;
        let z = x - (b[j] * mult[j] + b[i] * mult[i]) + (b[j] * mult[i] + b[i] * mult[j]);
        if h.insert(z) {
          w.push(z);
          if res > z {
            res = z;
          }
        }
      }
    }
    q = w;
  }
  unpack(res, n)
    .into_iter()
    .map(|i| (b'1' + i as u8) as char)
    .collect::<String>()
}

fn main() -> io::Result<()> {
  let stdin = io::stdin();
  let it = stdin.lock().lines().flat_map(|s| {
    s.unwrap()
      .split_whitespace()
      .map(|s| s.to_owned())
      .collect::<Vec<_>>()
      .into_iter()
  });
  let mut r = Scanner { it };
  let stdout = io::stdout();
  let mut o = io::BufWriter::with_capacity(1 << 20, stdout.lock());
  for _ in 0..r.next::<usize>() {
    let s = r
      .it
      .next()
      .unwrap()
      .chars()
      .map(|c| c as u8 - b'1')
      .collect::<Vec<_>>();
    let k = r.next::<i64>();
    writeln!(o, "{}", solve(&s, k))?;
  }
  o.flush()
}