//O(n ^ 2): naive/greedy solution (without FenwickTree) with heuristics if k >= number of invertion when output sorted string
use io::{BufRead, Write};
use std::io;
use std::str::FromStr;

struct Scanner<I> {
  it: I,
}

impl<I: std::iter::Iterator<Item = String>> Scanner<I> {
  fn next<A: FromStr>(&mut self) -> A {
    if let Ok(x) = self.it.next().unwrap().parse::<A>() {
      x
    } else {
      panic!("parse failed")
    }
  }
}

fn count_inversions(s: &[u8]) -> i64 {
  let mut a: [u32; 9] = [0; 9];
  let mut res = 0;
  for p in s.iter().rev() {
    let k = *p as usize;
    res += a.iter().take(k).sum::<u32>() as u64;
    a[k] += 1;
  }
  res as _
}

fn naive_solve(a: &mut [u8], mut k: i64) -> String {
  let n = a.len();
  let mut s = String::new();
  if k >= count_inversions(a) {
    a.sort();
    s.extend(a.iter().map(|d| (b'1' + *d) as char));
    return s;
  }
  for i in 0..n {
    let mut found = false;
    for d in 0..9 {
      if let Some(j) = a.iter().skip(i).position(|b| *b == d).map(|k| k + i) {
        let delta = (j - i) as i64;
        if delta <= k {
          for l in (i..j).rev() {
            a.swap(l, l + 1);
            k -= 1;
          }
          found = true;
          s.push((b'1' + d as u8) as char);
          break;
        }
      }
    }
    assert!(found);
  }
  assert_eq!(s.len(), n);
  s
}

fn main() -> io::Result<()> {
  let stdin = io::stdin();
  let it = stdin.lock().lines().flat_map(|s| {
    s.unwrap()
      .split_whitespace()
      .map(|s| s.to_owned())
      .collect::<Vec<_>>()
      .into_iter()
  });
  let mut r = Scanner { it };
  let stdout = io::stdout();
  let mut o = io::BufWriter::with_capacity(1 << 20, stdout.lock());
  for _ in 0..r.next::<usize>() {
    let mut s = r
      .it
      .next()
      .unwrap()
      .chars()
      .map(|c| c as u8 - b'1')
      .collect::<Vec<_>>();
    let k = r.next::<i64>();
    writeln!(o, "{}", naive_solve(&mut s, k))?;
  }
  o.flush()
}
