use io::{BufRead, Write};
use std::io;
use std::str::FromStr;

struct Scanner<I> {
  it: I,
}

impl<I: std::iter::Iterator<Item = String>> Scanner<I> {
  fn next<A: FromStr>(&mut self) -> A {
    if let Ok(x) = self.it.next().unwrap().parse::<A>() {
      x
    } else {
      panic!("parse failed")
    }
  }
}

pub struct FenwickTree<T> {
  a: Vec<T>,
  n: usize,
}

impl<T> FenwickTree<T>
where
  T: std::ops::AddAssign + Copy + From<bool>,
{
  pub fn new(n: usize) -> Self {
    Self {
      a: vec![T::from(false); n],
      n,
    }
  }
  pub fn update(&mut self, x: usize, v: T) {
    let n = self.n;
    let mut i = x;
    while i < n {
      self.a[i] += v;
      i |= i + 1;
    }
  }
  //sum on [0, x)
  pub fn reduce(&self, x: usize) -> T {
    let mut r = T::from(false);
    if x == 0 {
      return r;
    }
    let mut i = x - 1;
    loop {
      r += self.a[i];
      i &= i + 1;
      if i == 0 {
        break r;
      }
      i -= 1;
    }
  }
}

fn solve(a: &[u8], mut k: i64) -> String {
  let mut idx = vec![Vec::new(); 9];
  let mut off = vec![0u16; 9];
  for (i, p) in a.iter().enumerate() {
    idx[*p as usize].push(i);
  }
  let n = a.len();
  let mut ft: FenwickTree<i32> = FenwickTree::new(n);
  let mut used = vec![false; n];
  let mut s = String::new();
  s.reserve(n);
  let mut i = 0usize;
  for _t in 0..n {
    i += used.iter().skip(i).position(|b| !*b).unwrap();
    assert!(!used[i]);
    let cur = a[i] as usize;
    let before = ft.reduce(i) as usize;
    let f = |j: usize| -> i64 {
      assert!(j >= i);
      assert!(j < n);
      let between = ft.reduce(j + 1) as usize - before;
      let distance = j - i;
      assert!(distance >= between);
      (distance - between) as i64
    };
    let d = (0..9)
      .find(|&digit| {
        assert!(digit <= cur);
        idx[digit]
          .get(off[digit] as usize)
          .into_iter()
          .any(|j| f(*j) <= k)
      })
      .unwrap();
    let j = idx[d][off[d] as usize];
    k -= f(j);
    used[j] = true;
    ft.update(j, 1);
    off[d] += 1;
    s.push((b'1' + d as u8) as char);
  }
  s
}

fn main() -> io::Result<()> {
  let stdin = io::stdin();
  let it = stdin.lock().lines().flat_map(|s| {
    s.unwrap()
      .split_whitespace()
      .map(|s| s.to_owned())
      .collect::<Vec<_>>()
      .into_iter()
  });
  let mut r = Scanner { it };
  let stdout = io::stdout();
  let mut o = io::BufWriter::with_capacity(1 << 20, stdout.lock());
  for _ in 0..r.next::<usize>() {
    let s = r
      .it
      .next()
      .unwrap()
      .chars()
      .map(|c| c as u8 - b'1')
      .collect::<Vec<_>>();
    let k = r.next::<i64>();
    writeln!(o, "{}", solve(&s, k))?;
  }
  o.flush()
}
