// Author: Ivan Kazmenko (gassa@mail.ru)
// Generator for problem "read-ordinal"
module gen;
import std.algorithm;
import std.conv;
import std.exception;
import std.format;
import std.math;
import std.random;
import std.range;
import std.stdio;

immutable string testFormat = "%03d";
enum generateAnswers = false;

immutable int minN =  1;
immutable int maxN = 15;

struct Test
{
	string s;

	string comment;

	void validate () const
	{
		enforce (s.all !(q{"{,}".canFind (a)}));
	}

	void print (File f) const
	{
		f.writeln (s);
	}

	string log () const
	{
		return format !("%s (n = %s))")
		    (comment, ilogb (s.length / 2));
	}
}

Test [] tests;

void printAllTests ()
{
	foreach (testNumber, test; tests)
	{
		test.validate ();
		auto testString = format (testFormat, testNumber + 1);
		auto f = File (testString, "wt");
		test.print (f);
		static if (generateAnswers)
		{
			auto g = File (testString ~ ".a", "wt");
			test.printAnswer (g);
		}
		writeln (testString, ": ", test.log ());
	}
}

Mt19937 rndLocal;
T rndValue (T) () {return uniform !(T) (rndLocal);}
T rndValue (T) (T lim) {return uniform (cast (T) (0), lim, rndLocal);}
T rndValue (string boundaries = "[)", T) (T lo, T hi)
    {return uniform !(boundaries) (lo, hi, rndLocal);}
auto rndChoice (R) (R r) {return r[rndValue (cast (int) (r.length))];}

void rndShuffle (R) (R r)
{
	auto len = r.length.to !(int);
	foreach (i; 0..len)
	{
		int k = rndValue (i, len);
		r.swapAt (i, k);
	}
}

string genRandom (alias shuffle) (int n)
{
	auto elems = n.iota.map !(genRandom !(shuffle)).array;
	shuffle (elems);
	return "{" ~ elems.join (",") ~ "}";
}

void main ()
{
	rndLocal.seed (590_215_661);

	tests ~= Test ("{}", "example test 1");
	tests ~= Test ("{{}}", "example test 2");
	tests ~= Test ("{{},{{}}}", "example test 3");
	tests ~= Test ("{{{}},{{{}},{}},{}}", "example test 4");

	tests ~= Test ("{{{}},{}}", "alt test 1");

	foreach (cn; 4..15 + 1)
	{
		tests ~= Test (genRandom !(rndShuffle) (cn),
		    format !("general test %s") (cn - 4 + 1));
	}

	foreach (cn; 14..15 + 1)
	{
		tests ~= Test (genRandom !((a) {}) (cn),
		    format !("min-order test %s") (cn - 14 + 1));
		tests ~= Test (genRandom !(reverse) (cn),
		    format !("max-order test %s") (cn - 14 + 1));
	}

	printAllTests ();
}
